import React, { useContext } from 'react';
import Children from "./item";
import { getNormalDpi } from "../../utils/tools";
import "./index.less";
import { ExcelContext } from "../../index";
import { DataCompCxt } from "../index";
import { sumWidths } from "../../utils/tools";
import classnames from 'classnames';

// 构建表头, 需要显示的数据
var headerBuilder = function headerBuilder(minColumn, maxColumn, columnWidth, stickyHeight, noFixed, fixedLeft) {
  var fixedLeftLen = fixedLeft.data.length;
  var columns = [];
  var left = [0];
  var pos = 0;

  for (var c = 1; c <= maxColumn; c++) {
    pos += columnWidth(c - 1);
    left.push(pos);
  }

  for (var i = minColumn; i <= maxColumn; i++) {
    columns.push({
      style: {
        height: stickyHeight,
        width: columnWidth(i),
        left: left[i]
      },
      data: noFixed.data[i],
      index: fixedLeftLen + i
    });
  }

  return columns;
};

var dStyle = function dStyle(width, height) {
  var params = {};
  if (width) params.width = width;
  if (height) params.height = height;
  return params;
};

var StickyHeader = function StickyHeader(props) {
  var minColumn = props.minColumn,
      maxColumn = props.maxColumn;

  var _ref = useContext(ExcelContext),
      columnInfo = _ref.columnInfo,
      summation = _ref.summation,
      gridRef = _ref.gridRef,
      colData = _ref.colData;

  var _ref2 = useContext(DataCompCxt),
      columnWidth = _ref2.columnWidth;

  var height = colData.defaultItemHeight;
  var noFixed = columnInfo.noFixed,
      fixedLeft = columnInfo.fixedLeft,
      fixedRight = columnInfo.fixedRight,
      summationHeight = columnInfo.summationHeight,
      columns = columnInfo.columns;
  var stickyWidthLeft = fixedLeft.res;
  var stickyWidthRight = fixedRight.res;
  var stickyHeight = summationHeight;
  var centerColumns = headerBuilder(minColumn, maxColumn, columnWidth, stickyHeight, noFixed, fixedLeft);

  var fn = function fn(m) {
    if (m.width) {
      return m.width;
    } else if (m.children) {
      return sumWidths(m.children.map(function (v) {
        return v.width || 0;
      }));
    } else {
      return 0;
    }
  };

  var isEndBottom = function () {
    var isShowDirScrollBar = columnInfo.isShowDirScrollBar,
        dataSourceHeightInfo = columnInfo.dataSourceHeightInfo,
        contHeight = columnInfo.contHeight;
    var len = sumWidths(dataSourceHeightInfo);
    var maxHeight = len * height;
    var scrollTop = maxHeight - contHeight;

    if (isShowDirScrollBar) {
      return gridRef.current && Math.abs(scrollTop - gridRef.current.state.scrollTop) < 2;
    } else {
      return true;
    }
  }();

  var getNextColumns = function getNextColumns(curData) {
    var index = columns.findIndex(function (v) {
      return v.dataIndex === curData.dataIndex;
    });
    if (index === -1) return undefined;
    return columns[index + 1];
  };

  if (!summation) return null;
  return /*#__PURE__*/React.createElement("div", {
    className: classnames('salaryExcelSummationFooter', {
      isEndBottom: isEndBottom
    })
  }, /*#__PURE__*/React.createElement("div", {
    className: "baseLeft",
    style: dStyle(stickyWidthLeft, stickyHeight)
  }, fixedLeft.data.map(function (v, index) {
    return /*#__PURE__*/React.createElement(Children, {
      key: index,
      index: index,
      style: dStyle(fn(v)),
      mainColumns: v,
      nextColumns: getNextColumns(v),
      isFixCol: true
    });
  })), /*#__PURE__*/React.createElement("div", {
    className: "scrollable",
    style: dStyle(columnInfo.noFixed.res)
  }, centerColumns.map(function (_ref3, i) {
    var index = _ref3.index,
        data = _ref3.data,
        style = _ref3.style;
    return /*#__PURE__*/React.createElement("div", {
      className: "item",
      key: i,
      style: style
    }, /*#__PURE__*/React.createElement(Children, {
      key: data.dataIndex,
      index: index,
      style: dStyle(style.width),
      mainColumns: data,
      nextColumns: getNextColumns(data)
    }));
  })), /*#__PURE__*/React.createElement("div", {
    className: "baseRight",
    style: dStyle(stickyWidthRight - (getNormalDpi() ? 0 : 1), stickyHeight)
  }, fixedRight.data.map(function (v, index) {
    return /*#__PURE__*/React.createElement(Children, {
      key: index,
      index: index,
      style: dStyle(fn(v)),
      mainColumns: v,
      nextColumns: getNextColumns(v),
      isFixCol: true
    });
  })));
};

export default StickyHeader;